@@ -39,15 +39,15 @@ public:
   exp(position pos)
     : varinit(pos), ct(0) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) = 0;
 
   // When reporting errors with function calls, it is nice to say "no
   // function f(int)" instead of "no function matching signature
   // (int)."  Hence, this method returns the name of the expression if
   // there is one.
-  virtual symbol *getName()
+  virtual symbol getName()
   {
-    return 0;
+    return symbol::nullsym;
   }
 
   // Checks if the expression can be used as the right side of a scale
@@ -101,26 +101,62 @@ public:
   //      error.
   virtual types::ty *getType(coenv &) = 0;
 
+  // This is an optimization which works in some cases to by-pass the slow
+  // overloaded function resolution provided by the application class.
+  //
+  // If an expression is called with arguments given by sig, getCallee must
+  // either return 0 (the default), or if it returns a varEntry, the varEntry
+  // must correspond to the function which would be called after normal
+  // function resolution.
+  //
+  // The callee must produce no side effects as there are no guarantees when
+  // the varEntry will be translated.
+  virtual trans::varEntry *getCallee(coenv &e, types::signature *sig) {
+//#define DEBUG_GETAPP
+#if DEBUG_GETAPP
+    cout << "exp fail" << endl;
+    cout << "exp fail at " << getPos() << endl;
+    prettyprint(cout, 2);
+#endif
+    return 0;
+  }
+
   // Same result as getType, but caches the result so that subsequent
   // calls are faster.  For this to work correctly, the expression should
   // only be used in one place, so the environment doesn't change between
   // calls.
   virtual types::ty *cgetType(coenv &e) {
+#ifdef DEBUG_CACHE
+    testCachedType(e);
+#endif
     return ct ? ct : ct = getType(e);
   }
 
-  // The expression is being used as an address to write to.  This writes code
-  // so that the value on top of stack is put into the address (but not popped
-  // off the stack).
-  virtual void transWrite(coenv &, types::ty *) {
+  void testCachedType(coenv &e);
+
+  // The expression is being written.  Translate code such that the value
+  // (represented by the exp value) is stored into the address represented by
+  // this expression.
+  // In terms of side-effects, this expression must be evaluated (once) before
+  // value is evaluated (once).
+  virtual void transWrite(coenv &e, types::ty *t, exp *value) {
     em.error(getPos());
     em << "expression cannot be used as an address";
+
+    // Translate the value for errors.
+    value->transToType(e, t);
   }
 
   // Translates code for calling a function.  The arguments, in the order they
   // appear in the function's signature, must all be on the stack.
   virtual void transCall(coenv &e, types::ty *target);
 
+  // transConditionalJump must produce code equivalent to the following:
+  // Evaluate the expression as a boolean.  If the result equals cond, jump to
+  // the label dest, otherwise do not jump.  In either case, no value is left
+  // on the stack.
+  virtual void transConditionalJump(coenv &e, bool cond, Int dest);
+
   // This is used to ensure the proper order and number of evaluations.  When
   // called, it immediately translates code to perform the side-effects
   // consistent with a corresponding call to transAsType(e, target).
@@ -151,6 +187,8 @@ class tempExp : public exp {
 public:
   tempExp(coenv &e, varinit *v, types::ty *t);
 
+  void prettyprint(ostream &out, Int indent);
+
   types::ty *trans(coenv &e);
 
   types::ty *getType(coenv &) {
@@ -159,6 +197,7 @@ public:
 };
 
 // Wrap a varEntry so that it can be used as an expression.
+// Translating the varEntry must cause no side-effects.
 class varEntryExp : public exp {
   trans::varEntry *v;
 public:
@@ -167,12 +206,15 @@ public:
   varEntryExp(position pos, types::ty *t, access *a);
   varEntryExp(position pos, types::ty *t, vm::bltin f);
 
+  void prettyprint(ostream &out, Int indent);
+
   types::ty *getType(coenv &);
   types::ty *trans(coenv &e);
+  trans::varEntry *getCallee(coenv &e, types::signature *sig);
   
   void transAct(action act, coenv &e, types::ty *target);
   void transAsType(coenv &e, types::ty *target);
-  void transWrite(coenv &e, types::ty *target);
+  void transWrite(coenv &e, types::ty *t, exp *value);
   void transCall(coenv &e, types::ty *target);
 };
 
@@ -183,12 +225,15 @@ public:
   nameExp(position pos, name *value)
     : exp(pos), value(value) {}
 
-  nameExp(position pos, symbol *id)
+  nameExp(position pos, symbol id)
     : exp(pos), value(new simpleName(pos, id)) {}
 
+  nameExp(position pos, string s)
+    : exp(pos), value(new simpleName(pos, symbol::trans(s))) {}
+
   void prettyprint(ostream &out, Int indent);
 
-  symbol *getName()
+  symbol getName()
   {
     return value->getName();
   }
@@ -226,8 +271,16 @@ public:
     return t ? t : types::primError();
   }
 
-  void transWrite(coenv &e, types::ty *target) {
-    value->varTrans(trans::WRITE, e, target);
+  trans::varEntry *getCallee(coenv &e, types::signature *sig) {
+#ifdef DEBUG_GETAPP
+    cout << "nameExp" << endl;
+#endif
+    return value->getCallee(e, sig);
+  }
+
+  void transWrite(coenv &e, types::ty *target, exp *newValue) {
+    newValue->transToType(e, target);
+    this->value->varTrans(trans::WRITE, e, target);
 
     ct=0;  // See note in transAsType.
   }
@@ -248,7 +301,7 @@ public:
 // like f().x or (new t).x, a separate expression is needed.
 class fieldExp : public nameExp {
   exp *object;
-  symbol *field;
+  symbol field;
 
   // fieldExp has a lot of common functionality with qualifiedName, so we
   // essentially hack qualifiedName, by making our object expression look
@@ -268,6 +321,12 @@ class fieldExp : public nameExp {
     types::ty *varGetType(coenv &e) {
       return object->getType(e);
     }
+    trans::varEntry *getCallee(coenv &, types::signature *) {
+#ifdef DEBUG_GETAPP
+      cout << "pseudoName" << endl;
+#endif
+      return 0;
+    }
 
     // As a type:
     types::ty *typeTrans(coenv &, bool tacit = false) {
@@ -298,7 +357,7 @@ class fieldExp : public nameExp {
     void print(ostream& out) const {
       out << "<exp>";
     }
-    symbol *getName() {
+    symbol getName() {
       return object->getName();
     }
   };
@@ -307,7 +366,7 @@ class fieldExp : public nameExp {
   types::ty *getObject(coenv &e);
 
 public:
-  fieldExp(position pos, exp *object, symbol *field)
+  fieldExp(position pos, exp *object, symbol field)
     : nameExp(pos, new qualifiedName(pos,
                                      new pseudoName(object),
                                      field)),
@@ -315,7 +374,7 @@ public:
 
   void prettyprint(ostream &out, Int indent);
 
-  symbol *getName()
+  symbol getName()
   {
     return field;
   }
@@ -352,7 +411,7 @@ public:
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
-  void transWrite(coenv &e, types::ty *target);
+  void transWrite(coenv &e, types::ty *t, exp *value);
 
   exp *evaluate(coenv &e, types::ty *) {
     return new subscriptExp(getPos(),
@@ -397,7 +456,7 @@ public:
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
-  void transWrite(coenv &e, types::ty *target);
+  void transWrite(coenv &e, types::ty *t, exp *value);
 
   exp *evaluate(coenv &e, types::ty *) {
     return new sliceExp(getPos(),
@@ -417,6 +476,11 @@ public:
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
+
+  exp *evaluate(coenv &, types::ty *) {
+    // this has no side-effects
+    return this;
+  }
 };
 
 class literalExp : public exp {
@@ -425,6 +489,11 @@ public:
     : exp(pos) {}
 
   bool scalable() { return false; }
+
+  exp *evaluate(coenv &, types::ty *) {
+    // Literals are constant, they have no side-effects.
+    return this;
+  }
 };
 
 class intExp : public literalExp {
@@ -466,6 +535,8 @@ public:
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primString(); }
+
+  const string& getString() { return str; }
 };
 
 class booleanExp : public literalExp {
@@ -570,18 +641,15 @@ public:
 
 struct argument {
   exp *val;
-  symbol *name;
+  symbol name;
 
+  // No constructor due to the union in camp.y
 #if 0
-  argument(exp *val=0, symbol *name=0)
+  argument(exp *val=0, symbol name=0)
     : val(val), name(name) {}
 #endif
 
   void prettyprint(ostream &out, Int indent);
-
-  // Tests if a named argument could be mistaken for an assignment, and
-  // prints a warning if so.
-  void assignAmbiguity(coenv &e);
 };
 
 class arglist : public gc {
@@ -599,7 +667,7 @@ public:
     args.insert(args.begin(), a);
   }
 
-  virtual void addFront(exp *val, symbol *name=0) {
+  virtual void addFront(exp *val, symbol name=symbol::nullsym) {
     argument a; a.val=val; a.name=name;
     addFront(a);
   }
@@ -608,7 +676,7 @@ public:
     args.push_back(a);
   }
 
-  virtual void add(exp *val, symbol *name=0) {
+  virtual void add(exp *val, symbol name=symbol::nullsym) {
     argument a; a.val=val; a.name=name;
     add(a);
   }
@@ -619,11 +687,11 @@ public:
     return args.size();
   }
   
-  virtual argument operator[] (size_t index) {
+  virtual argument& operator[] (size_t index) {
     return args[index];
   }
 
-  virtual argument getRest() {
+  virtual argument& getRest() {
     return rest;
   }
 };
@@ -639,12 +707,14 @@ protected:
 
 private:
   // Per object caching - Cache the application when it's determined.
-  application *ca;
+  application *cachedApp;
 
-  // Warns of ambiguity with assign expression in named arguments.
-  void argAmbiguity(coenv &e);
+  // In special cases, no application object is needed and we can store the
+  // varEntry used in advance.
+  trans::varEntry *cachedVarEntry;
 
-  types::signature *argTypes(coenv& e);
+  types::signature *argTypes(coenv& e, bool *searchable);
+  void reportArgErrors(coenv &e);
   application *resolve(coenv &e,
                        types::overloaded *o,
                        types::signature *source,
@@ -653,42 +723,59 @@ private:
                                 types::overloaded *o,
                                 types::signature *source,
                                 bool tacit);
-  void reportMismatch(symbol *s,
-                      types::function *ft,
+  void reportMismatch(types::function *ft,
                       types::signature *source);
-  application *getApplication(coenv &e);
 
+  void reportNonFunction();
+
+  // Caches either the application object used to apply the function to the
+  // arguments, or in cases where the arguments match the function perfectly,
+  // the varEntry of the callee (or neither in case of an error).  Returns
+  // what getType should return.
+  types::ty *cacheAppOrVarEntry(coenv &e, bool tacit);
+
+  types::ty *transPerfectMatch(coenv &e);
 public:
   callExp(position pos, exp *callee, arglist *args)
-    : exp(pos), callee(callee), args(args), ca(0) { assert(args); }
+    : exp(pos), callee(callee), args(args),
+      cachedApp(0), cachedVarEntry(0) { assert(args); }
 
   callExp(position pos, exp *callee)
-    : exp(pos), callee(callee), args(new arglist()), ca(0) {}
+    : exp(pos), callee(callee), args(new arglist()),
+      cachedApp(0), cachedVarEntry(0) {}
 
   callExp(position pos, exp *callee, exp *arg1)
-    : exp(pos), callee(callee), args(new arglist()), ca(0) {
-      args->add(arg1);
-    }
+    : exp(pos), callee(callee), args(new arglist()),
+      cachedApp(0), cachedVarEntry(0) {
+    args->add(arg1);
+  }
 
   callExp(position pos, exp *callee, exp *arg1, exp *arg2)
-    : exp(pos), callee(callee), args(new arglist()), ca(0) {
-      args->add(arg1);
-      args->add(arg2);
-    }
+    : exp(pos), callee(callee), args(new arglist()),
+      cachedApp(0), cachedVarEntry(0) {
+    args->add(arg1);
+    args->add(arg2);
+  }
 
   callExp(position pos, exp *callee, exp *arg1, exp *arg2, exp *arg3)
-    : exp(pos), callee(callee), args(new arglist()), ca(0) {
-      args->add(arg1);
-      args->add(arg2);
-      args->add(arg3);
-    }
+    : exp(pos), callee(callee), args(new arglist()),
+      cachedApp(0), cachedVarEntry(0) {
+    args->add(arg1);
+    args->add(arg2);
+    args->add(arg3);
+  }
 
   void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
+
+  // Returns true if the function call resolves uniquely without error.  Used
+  // in implementing the special == and != operators for functions.
+  virtual bool resolved(coenv &e);
 };
 
+
 class pairExp : public exp {
   exp *x;
   exp *y;
@@ -725,7 +812,7 @@ class transformExp : public exp {
 
 public:
   transformExp(position pos, exp *x, exp *y, exp *xx, exp *xy, exp *yx,
-	       exp *yy)
+               exp *yy)
     : exp(pos), x(x), y(y), xx(xx), xy(xy), yx(yx), yy(yy) {}
 
   void prettyprint(ostream &out, Int indent);
@@ -739,7 +826,7 @@ class castExp : public exp {
   exp *castee;
 
   types::ty *tryCast(coenv &e, types::ty *t, types::ty *s,
-                     symbol *csym);
+                     symbol csym);
 public:
   castExp(position pos, ty *target, exp *castee)
     : exp(pos), target(target), castee(castee) {}
@@ -752,22 +839,36 @@ public:
 
 class nullaryExp : public callExp {
 public:
-  nullaryExp(position pos, symbol *op)
+  nullaryExp(position pos, symbol op)
     : callExp(pos, new nameExp(pos, op)) {}
 };
 
 class unaryExp : public callExp {
 public:
-  unaryExp(position pos, exp *base, symbol *op)
+  unaryExp(position pos, exp *base, symbol op)
     : callExp(pos, new nameExp(pos, op), base) {}
 };
 
 class binaryExp : public callExp {
 public:
-  binaryExp(position pos, exp *left, symbol *op, exp *right)
+  binaryExp(position pos, exp *left, symbol op, exp *right)
     : callExp(pos, new nameExp(pos, op), left, right) {}
 };
 
+class equalityExp : public callExp {
+public:
+  equalityExp(position pos, exp *left, symbol op, exp *right)
+    : callExp(pos, new nameExp(pos, op), left, right) {}
+
+  void prettyprint(ostream &out, Int indent);
+
+#ifdef NO_FUNC_OPS
+  types::ty *trans(coenv &e);
+  types::ty *getType(coenv &e);
+#endif
+};
+
+
 // Scaling expressions such as 3sin(x).
 class scaleExp : public binaryExp {
   exp *getLeft() {
@@ -792,7 +893,7 @@ public:
 // is a tension atleast case.
 class ternaryExp : public callExp {
 public:
-  ternaryExp(position pos, exp *left, symbol *op, exp *right, exp *last)
+  ternaryExp(position pos, exp *left, symbol op, exp *right, exp *last)
     : callExp(pos, new nameExp(pos, op), left, right, last) {}
 };
 
@@ -819,11 +920,11 @@ public:
 class andOrExp : public exp {
 protected:
   exp *left;
-  symbol *op;
+  symbol op;
   exp *right;
 
 public:
-  andOrExp(position pos, exp *left, symbol *op, exp *right)
+  andOrExp(position pos, exp *left, symbol op, exp *right)
     : exp(pos), left(left), op(op), right(right) {}
 
   virtual types::ty *trans(coenv &e) = 0;
@@ -834,27 +935,29 @@ public:
 
 class orExp : public andOrExp {
 public:
-  orExp(position pos, exp *left, symbol *op, exp *right)
+  orExp(position pos, exp *left, symbol op, exp *right)
     : andOrExp(pos, left, op, right) {}
 
   void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
+  void transConditionalJump(coenv &e, bool cond, Int dest);
 };
 
 class andExp : public andOrExp {
 public:
-  andExp(position pos, exp *left, symbol *op, exp *right)
+  andExp(position pos, exp *left, symbol op, exp *right)
     : andOrExp(pos, left, op, right) {}
 
   void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
+  void transConditionalJump(coenv &e, bool cond, Int dest);
 };
 
 class joinExp : public callExp {
 public:
-  joinExp(position pos, symbol *op)
+  joinExp(position pos, symbol op)
     : callExp(pos, new nameExp(pos, op)) {}
 
   void pushFront(exp *e) {
@@ -868,12 +971,12 @@ public:
 };
 
 class specExp : public exp {
-  symbol *op;
+  symbol op;
   exp *arg;
   camp::side s;
 
 public:
-  specExp(position pos, symbol *op, exp *arg, camp::side s=camp::OUT)
+  specExp(position pos, symbol op, exp *arg, camp::side s=camp::OUT)
     : exp(pos), op(op), arg(arg), s(s) {}
 
   void setSide(camp::side ss) {
@@ -913,25 +1016,27 @@ public:
 };
 
 class selfExp : public assignExp {
-  symbol *op;
+  symbol op;
 
   exp *ultimateValue(exp *dest) {
     return new binaryExp(getPos(), dest, op, value);
   }
 
 public:
-  selfExp(position pos, exp *dest, symbol *op, exp *value)
+  selfExp(position pos, exp *dest, symbol op, exp *value)
     : assignExp(pos, dest, value), op(op) {}
 
   void prettyprint(ostream &out, Int indent);
+
+  void transAsType(coenv &e, types::ty *target);
 };
 
 class prefixExp : public exp {
   exp *dest;
-  symbol *op;
+  symbol op;
 
 public:
-  prefixExp(position pos, exp *dest, symbol *op)
+  prefixExp(position pos, exp *dest, symbol op)
     : exp(pos), dest(dest), op(op) {}
 
   void prettyprint(ostream &out, Int indent);
@@ -950,10 +1055,10 @@ public:
 // error."
 class postfixExp : public exp {
   exp *dest;
-  symbol *op;
+  symbol op;
 
 public:
-  postfixExp(position pos, exp *dest, symbol *op)
+  postfixExp(position pos, exp *dest, symbol op)
     : exp(pos), dest(dest), op(op) {}
 
   void prettyprint(ostream &out, Int indent);
