@@ -1,10 +1,6 @@
 #ifndef FPU_H
 #define FPU_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifdef HAVE_FENV_H
 #ifdef _GNU_SOURCE
 #define HAVE_FEENABLEEXCEPT
@@ -14,8 +10,8 @@
 #ifdef HAVE_FEENABLEEXCEPT
 #include <fenv.h>
 
-inline int fpu_exceptions() {
-  int excepts=0;
+inline Int fpu_exceptions() {
+  Int excepts=0;
 #ifdef FE_INVALID    
   excepts |= FE_INVALID;
 #endif    
