@@ -110,20 +110,24 @@ void formals::addOps(coenv &e, record *r)
 // Another helper class. Does an assignment, but relying only on the
 // destination for the type.
 class basicAssignExp : public exp {
-  exp *dest;
+  varEntry *dest;
   varinit *value;
 public:
-  basicAssignExp(position pos, exp *dest, varinit *value) 
+  basicAssignExp(position pos, varEntry *dest, varinit *value) 
     : exp(pos), dest(dest), value(value) {}
 
-  types::ty *getType(coenv &e) {
-    return dest->getType(e);
+  void prettyprint(ostream &out, Int indent) {
+    prettyname(out, "basicAssignExp", indent);
+  }
+
+  types::ty *getType(coenv &) {
+    return dest->getType();
   }
 
   types::ty *trans(coenv &e) {
     // This doesn't handle overloaded types for the destination.
     value->transToType(e, getType(e));
-    dest->transWrite(e, getType(e));
+    dest->encode(WRITE, pos, e.c);
     return getType(e);
   }
 };
@@ -143,13 +147,13 @@ void transDefault(coenv &e, position pos, varEntry *v, varinit *init) {
                        &vee),
            new expStm(pos,
                       new basicAssignExp(pos,
-                                         &vee,
+                                         v,
                                          init)));
   is.trans(e);                                        
 }
 
 void formal::transAsVar(coenv &e, Int index) {
-  symbol *name = getName();
+  symbol name = getName();
   if (name) {
     trans::access *a = e.c.accessFormal(index);
     assert(a);
@@ -230,8 +234,10 @@ varinit *fundef::makeVarInit(function *ft) {
 
 void fundef::baseTrans(coenv &e, types::function *ft)
 {
+  string name = id ? string(id) : string("<anonymous function>");
+
   // Create a new function environment.
-  coder fc = e.c.newFunction(ft);
+  coder fc = e.c.newFunction(getPos(), name, ft);
   coenv fe(fc,e.e);
 
   // Translate the function.
@@ -284,7 +290,7 @@ types::ty *fundef::trans(coenv &e) {
 void fundec::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
-  out << "fundec '" << *id << "'\n";
+  out << "fundec '" << id << "'\n";
 
   fun.prettyprint(out, indent);
 }
