@@ -9,8 +9,14 @@
 #define ITEM_H
 
 #include "common.h"
-#include <typeinfo>
+#include <cfloat>
+#include <cmath>
 
+#if COMPACT
+#include <cassert>
+#else
+#include <typeinfo>
+#endif
 
 namespace vm {
 
@@ -20,18 +26,95 @@ class bad_item_value {};
 template<typename T>
 T get(const item&);
 
+#if COMPACT
+// Identify a default argument.
+extern const Int DefaultValue;
+  
+// Identify an undefined item.
+extern const Int Undefined;
+
+// Values for true and false unlikely to match other values.
+extern const Int BoolTruthValue;
+extern const Int BoolFalseValue;
+
+inline Int valueFromBool(bool b) {
+  return b ? BoolTruthValue : BoolFalseValue;
+}
+#endif
+  
+extern const item Default;
+
 class item : public gc {
+private:
+  
+#if !COMPACT
+  const std::type_info *kind;
+#endif
+  
+  union {
+    Int i;
+    double x;
+#if !COMPACT
+    bool b;
+#endif
+    void *p;
+  };
+
 public:
-  bool empty()
-  { return *kind == typeid(void); }
+#if COMPACT    
+  bool empty() const
+  {return i >= Undefined;}
+  
+  item() : i(Undefined) {}
+  
+  item(Int i)
+    : i(i) {}
+  item(int i)
+    : i(i) {}
+  item(double x)
+    : x(x) {}
+  item(bool b)
+    : i(valueFromBool(b)) {}
+  
+  item& operator= (int a)
+  { i=a; return *this; }
+  item& operator= (unsigned int a)
+  { i=a; return *this; }
+  item& operator= (Int a)
+  { i=a; return *this; }
+  item& operator= (double a)
+  { x=a; return *this; }
+  item& operator= (bool b)
+  { i=valueFromBool(b); return *this; }
+  
+  template<class T>
+  item(T *p)
+    : i((Int) p) {
+    assert(!empty());
+  }
+  
+  template<class T>
+  item(const T &p)
+    : i((Int) new(UseGC) T(p)) {
+    assert(!empty());
+  }
+  
+  template<class T>
+  item& operator= (T *a)
+  { i=(Int) a; return *this; }
+  
+  template<class T>
+  item& operator= (const T &it)
+  { i=(Int) new(UseGC) T(it); return *this; }
+#else    
+  bool empty() const
+  {return *kind == typeid(void);}
   
   item()
     : kind(&typeid(void)) {}
   
-#ifndef Int  
   item(Int i)
     : kind(&typeid(Int)), i(i) {}
-#endif  
   item(int i)
     : kind(&typeid(Int)), i(i) {}
   item(double x)
@@ -39,10 +122,10 @@ public:
   item(bool b)
     : kind(&typeid(bool)), b(b) {}
   
-#ifndef Int  
   item& operator= (int a)
   { kind=&typeid(Int); i=a; return *this; }
-#endif  
+  item& operator= (unsigned int a)
+  { kind=&typeid(Int); i=a; return *this; }
   item& operator= (Int a)
   { kind=&typeid(Int); i=a; return *this; }
   item& operator= (double a)
@@ -66,28 +149,18 @@ public:
   item& operator= (const T &it)
   { kind=&typeid(T); p=new(UseGC) T(it); return *this; }
   
+  const std::type_info &type() const
+  { return *kind; }
+#endif  
+  
   template<typename T>
   friend inline T get(const item&);
 
   friend inline bool isdefault(const item&);
-  friend inline bool isarray(const item&);
   
-  const std::type_info &type() const
-  { return *kind; }
-
   friend ostream& operator<< (ostream& out, const item& i);
 
 private:
-  
-  const std::type_info *kind;
-  
-  union {
-    Int i;
-    double x;
-    bool b;
-    void *p;
-  };
-
   template <typename T>
   struct help;
   
@@ -95,8 +168,13 @@ private:
   struct help<T*> {
     static T* unwrap(const item& it)
     {
-      if (*it.kind == typeid(T))
+#if COMPACT      
+      if(!it.empty())
         return (T*) it.p;
+#else        
+      if(*it.kind == typeid(T))
+        return (T*) it.p;
+#endif        
       throw vm::bad_item_value();
     }
   };
@@ -105,20 +183,36 @@ private:
   struct help {
     static T& unwrap(const item& it)
     {
-      if (*it.kind == typeid(T))
+#if COMPACT      
+      if(!it.empty())
         return *(T*) it.p;
+#else      
+      if(*it.kind == typeid(T))
+        return *(T*) it.p;
+#endif      
       throw vm::bad_item_value();
     }
   };
 };
   
 class frame : public gc {
+#ifdef DEBUG_FRAME
+  string name;
+#endif
   typedef mem::vector<item> vars_t;
   vars_t vars;
 public:
+#ifdef DEBUG_FRAME
+  frame(string name, size_t size)
+    : name(name), vars(size)
+  {}
+
+  string getName() { return name; }
+#else
   frame(size_t size)
     : vars(size)
   {}
+#endif
 
   item& operator[] (size_t n)
   { return vars[n]; }
@@ -130,7 +224,7 @@ public:
   
   // Extends vars to ensure it has a place for any variable indexed up to n.
   void extend(size_t n) {
-    if (vars.size() < n)
+    if(vars.size() < n)
       vars.resize(n);
   }
 };
@@ -141,61 +235,69 @@ inline T get(const item& it)
   return item::help<T>::unwrap(it);
 } 
 
-#ifndef Int  
 template <>
 inline int get<int>(const item&)
 {
   throw vm::bad_item_value();
 }
-#endif
   
 template <>
 inline Int get<Int>(const item& it)
 {
-  if (*it.kind == typeid(Int))
+#if COMPACT  
+  if(!it.empty())
     return it.i;
+#else
+  if(*it.kind == typeid(Int))
+    return it.i;
+#endif  
   throw vm::bad_item_value();
 }
   
 template <>
 inline double get<double>(const item& it)
 {
-  if (*it.kind == typeid(double))
+#if COMPACT  
+  if(!it.empty())
     return it.x;
+#else
+  if(*it.kind == typeid(double))
+    return it.x;
+#endif
   throw vm::bad_item_value();
 }
 
 template <>
 inline bool get<bool>(const item& it)
 {
-  if (*it.kind == typeid(bool))
+#if COMPACT  
+  if(it.i == BoolTruthValue)
+    return true;
+  if(it.i == BoolFalseValue)
+    return false;
+#else  
+  if(*it.kind == typeid(bool))
     return it.b;
+#endif  
   throw vm::bad_item_value();
 }
 
+#if !COMPACT
 // This serves as the object for representing a default argument.
 struct default_t : public gc {};
-  
+#endif
+
 inline bool isdefault(const item& it)
 {
+#if COMPACT  
+  return it.i == DefaultValue;
+#else  
   return *it.kind == typeid(default_t);
+#endif  
 } 
 
-inline ostream& operator<< (ostream& out, const item& i)
-{
-  out << "type " << i.type().name();
-  if (i.type() == typeid(Int))
-    cout << ", value = " << get<Int>(i) << endl;
-  else if (i.type() == typeid(double))
-    cout << ", value = " << get<double>(i) << endl;
-  else if (i.type() == typeid(string))
-    cout << ", value = " << get<string>(i) << endl;
-  else out << endl;
-  return out;
-}
+ostream& operator<< (ostream& out, const item& i);
 
 } // namespace vm
 
-GC_DECLARE_PTRFREE(vm::default_t);
-
 #endif // ITEM_H
