@@ -14,11 +14,12 @@
 #include <string>
 #include <sstream>
 
-#ifndef NOHASH
 #ifndef __GNUC_PREREQ
 #define __GNUC_PREREQ(maj, min) (0)
-# endif
-#if __GNUC_PREREQ(4,3)
+#endif
+
+#ifndef NOHASH
+#if __GNUC_PREREQ(4,3) || defined(__CYGWIN__)
 #include <tr1/unordered_map>
 #define EXT std::tr1
 #else
@@ -37,6 +38,7 @@
   
 #ifdef USEGC
 
+#define GC_THREADS
 #include <gc.h>
 
 #ifdef GC_DEBUG
@@ -50,9 +52,9 @@ inline void *asy_malloc(size_t n)
 #ifdef GC_DEBUG
   if(void *mem=GC_debug_malloc_ignore_off_page(n, GC_EXTRAS))
 #else
-  if(void *mem=GC_malloc_ignore_off_page(n))
+    if(void *mem=GC_malloc_ignore_off_page(n))
 #endif
-    return mem;
+      return mem;
   throw std::bad_alloc();
 }
 
@@ -61,9 +63,9 @@ inline void *asy_malloc_atomic(size_t n)
 #ifdef GC_DEBUG
   if(void *mem=GC_debug_malloc_atomic_ignore_off_page(n, GC_EXTRAS))
 #else
-  if(void *mem=GC_malloc_atomic_ignore_off_page(n))
+    if(void *mem=GC_malloc_atomic_ignore_off_page(n))
 #endif
-    return mem;
+      return mem;
   throw std::bad_alloc();
 }
 
@@ -97,19 +99,19 @@ inline void* operator new[](size_t size, GCPlacement) {
 template<class T>
 struct GC_type_traits {};
 
-#define GC_DECLARE_PTRFREE(T) \
-template<> struct GC_type_traits<T> {}
+#define GC_DECLARE_PTRFREE(T)                   \
+  template<> struct GC_type_traits<T> {}
 
 #endif // USEGC
 
 namespace mem {
 
-#define GC_CONTAINER(KIND)                                               \
-  template <typename T>                                                  \
-  struct KIND : public std::KIND<T, gc_allocator<T> >, public gc {       \
-    KIND() : std::KIND<T, gc_allocator<T> >() {}                         \
-    KIND(size_t n) : std::KIND<T, gc_allocator<T> >(n) {}                \
-    KIND(size_t n, const T& t) : std::KIND<T, gc_allocator<T> >(n,t) {}  \
+#define GC_CONTAINER(KIND)                                              \
+  template <typename T>                                                 \
+  struct KIND : public std::KIND<T, gc_allocator<T> >, public gc {      \
+    KIND() : std::KIND<T, gc_allocator<T> >() {}                        \
+    KIND(size_t n) : std::KIND<T, gc_allocator<T> >(n) {}               \
+    KIND(size_t n, const T& t) : std::KIND<T, gc_allocator<T> >(n,t) {} \
   }
 
 GC_CONTAINER(list);
@@ -121,11 +123,11 @@ struct stack : public std::stack<T, Container>, public gc {
 
 #undef GC_CONTAINER
 
-#define GC_CONTAINER(KIND)                                                    \
-  template <typename Key, typename T, typename Compare = std::less<Key> >     \
-  struct KIND : public                                                        \
-  std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > >, public gc {      \
-    KIND() : std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > > () {}  \
+#define GC_CONTAINER(KIND)                                              \
+  template <typename Key, typename T, typename Compare = std::less<Key> > \
+  struct KIND : public                                                  \
+  std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > >, public gc { \
+    KIND() : std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > > () {} \
   }
 
 GC_CONTAINER(map);
@@ -134,15 +136,15 @@ GC_CONTAINER(multimap);
 #undef GC_CONTAINER
 
 #ifndef NOHASH
-#define GC_CONTAINER(KIND)                                                    \
-  template <typename Key, typename T,                                         \
-            typename Hash = EXT::hash<Key>,                                   \
-            typename Eq = std::equal_to<Key> >                                \
-  struct KIND : public                                                        \
-  EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > >, public gc {     \
+#define GC_CONTAINER(KIND)                                              \
+  template <typename Key, typename T,                                   \
+            typename Hash = EXT::hash<Key>,                             \
+            typename Eq = std::equal_to<Key> >                          \
+  struct KIND : public                                                  \
+  EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > >, public gc { \
     KIND() : EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > > () {} \
-    KIND(size_t n)                                                            \
-      : EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > > (n) {}     \
+    KIND(size_t n)                                                      \
+      : EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > > (n) {} \
   }
 
 GC_CONTAINER(unordered_map);
@@ -155,6 +157,8 @@ GC_CONTAINER(unordered_multimap);
 #ifdef USEGC
 typedef std::basic_string<char,std::char_traits<char>,
                           gc_allocator<char> > string;
+typedef std::basic_stringstream<char,std::char_traits<char>,
+                                gc_allocator<char> > stringstream;
 typedef std::basic_istringstream<char,std::char_traits<char>,
                                  gc_allocator<char> > istringstream;
 typedef std::basic_ostringstream<char,std::char_traits<char>,
@@ -163,6 +167,7 @@ typedef std::basic_stringbuf<char,std::char_traits<char>,
                              gc_allocator<char> > stringbuf;
 #else
 typedef std::string string;
+typedef std::stringstream stringstream;
 typedef std::istringstream istringstream;
 typedef std::ostringstream ostringstream;
 typedef std::stringbuf stringbuf;
