@@ -18,7 +18,12 @@ static const char* opnames[] = {
   "varpush", "varsave", "fieldpush", "fieldsave",
   "builtin", "jmp", "cjmp", "njmp", "call",
   "pushclosure", "makefunc", "ret",
-  "alloc", "pushframe", "popframe"
+  "alloc", "pushframe", "popframe",
+
+#ifdef COMBO
+  "varpop", "fieldpop",
+  "gejmp"
+#endif
 };
 static const Int numOps = (Int)(sizeof(opnames)/sizeof(char *));
 
@@ -36,6 +41,7 @@ string lookupBltin(bltin b) {
 
 ostream& operator<< (ostream& out, const item& i)
 {
+/*
   // TODO: Make a data structure mapping typeids to print functions.
   if (i.empty())
     out << "empty";
@@ -57,6 +63,7 @@ ostream& operator<< (ostream& out, const item& i)
   }
   else
     out << "type " << demangle(i.type().name());
+*/
   return out;
 }
 
@@ -80,6 +87,10 @@ void printInst(ostream& out, const program::label& code,
     case inst::fieldpush:
     case inst::fieldsave:
     case inst::alloc:
+#ifdef COMBO
+    case inst::varpop:
+    case inst::fieldpop:
+#endif
     {
       out << " " << get<Int>(*code);
       break;
@@ -104,6 +115,9 @@ void printInst(ostream& out, const program::label& code,
     case inst::jmp:
     case inst::cjmp:
     case inst::njmp:
+#ifdef COMBO
+    case inst::gejmp:
+#endif
     {
       char f = out.fill('0');
       out << " i";
