@@ -21,6 +21,8 @@ public:
   void encode(inst i);
   label begin();
   label end();
+  inst &back();
+  void pop_back();
 private:
   friend class label;
   typedef mem::vector<inst> code_t;
@@ -52,7 +54,7 @@ private:
 // Prints one instruction (including arguments) and returns how many
 // positions in the code stream were shown.
 void printInst(std::ostream& out, const program::label& code,
-	       const program::label& base);
+               const program::label& base);
 
 // Prints code until a ret opcode is printed.
 void print(std::ostream& out, program *base);
@@ -64,6 +66,10 @@ inline program::label program::end()
 { return label(code.size(), this); }
 inline program::label program::begin()
 { return label(0, this); }
+inline inst& program::back()
+{ return code.back(); }
+inline void program::pop_back()
+{ return code.pop_back(); }
 inline void program::encode(inst i)
 { code.push_back(i); }
 inline inst& program::operator[](size_t n)
